//-------------------------------------------------------------------------------------------------------
//
// Konfiguration
//
//-------------------------------------------------------------------------------------------------------


#ifndef _Config_H
#define _Config_H


#include <avr/io.h>


//-------------------------------------------------------------------------------------------------------
// CPU Takt

#ifndef F_CPU
#define F_CPU	20000000UL
#endif


//-------------------------------------------------------------------------------------------------------
// Makros

#define	RESET(x)		_XRS(x)
#define	SET(x)			_XS(x)
#define	TOGGLE(x)		_XT(x)
#define	SET_OUTPUT(x)	_XSO(x)
#define	SET_INPUT(x)	_XSI(x)
#define	IS_SET(x)		_XR(x)

#define	PORT(x)			_port2(x)
#define	DDR(x)			_ddr2(x)
#define	PIN(x)			_pin2(x)

#define	_XRS(x,y)		PORT(x) &= ~(1<<y)
#define	_XS(x,y)		PORT(x) |= (1<<y)
#define	_XT(x,y)		PORT(x) ^= (1<<y)

#define	_XSO(x,y)		DDR(x) |= (1<<y)
#define	_XSI(x,y)		DDR(x) &= ~(1<<y)
#define	_XR(x,y)		((PIN(x) & (1<<y)) != 0)

#define	_port2(x)		PORT ## x
#define	_ddr2(x)		DDR ## x
#define	_pin2(x)		PIN ## x


//-------------------------------------------------------------------------------------------------------
// CAN Adressen

#define CAN_Adresse			11

#define EGW					9

#define KU_EC				30
#define	DL_EC				31
#define	WZ_EC				32
#define WZ_EC_TV			33
#define BD_EC				34
#define	SZ_EC				35
#define	K1_EC				36
#define	K2_EC				37
#define	K3_EC				38
#define SH_EC				39

#define	KU_LM_West			60
#define	KU_LM_Sued			61
#define	DL_LM				62
#define	WZ_LM				63
#define	BD_LM				64
#define	GA_LM				65
#define	SZ_LM				66
#define	SH_LM				67


//-------------------------------------------------------------------------------------------------------
// Ein- und Ausgnge

// SPI
#define	P_MOSI			B,5
#define	P_MISO			B,6
#define	P_SCK			B,7
#define	MCP2515_SS		B,4

// Interrupts
#define	MCP2515_INT		B,2		// INT2

// Statusled
#define LED_G			C,2		// grn
#define LED_R			C,3		// rot

// 24V Ausgnge
#define DA1				D,0
#define DA2				D,1
#define DA3				D,2
#define DA4				D,3
#define DA5				D,4
#define DA6				D,5
#define DA7				D,6
#define DA8				D,7

// 24V Eingnge
#define DE1				0x0001
#define DE2				0x0002
#define DE3				0x0004
#define DE4				0x0008
#define DE5				0x0010
#define DE6				0x0020
#define DE7				0x0040
#define DE8				0x0080
#define DE9				0x0100
#define DE10			0x0200
#define DE11			0x2000
#define DE12			0x1000
#define DE13			0x0800
#define DE14			0x0400


//-------------------------------------------------------------------------------------------------------
// globale Variablen

// Uhrzeit und Datum
volatile uint8_t Ms10;                // 0-99
volatile uint8_t Sekunde;             // 0-59
volatile uint8_t Minute;              // 0-59
volatile uint8_t Stunde;              // 0-23
volatile uint8_t Tag;                 // 1-31
volatile uint8_t Monat;               // 1-12
volatile uint8_t Jahr;                // Wert + 2000 rechnen
volatile uint8_t Wochentag;           // 1=MO, 2=DI, 3=MI, 4=DO, 5=FR, 6=SA, 7=SO


// DE Auswertung
#define		DE_Zeit_kurz		50		// Zeitdauer fr 2x drcken (x 10ms), muss kleiner als bei der Wiederholfunktion sein
#define		DE_Zeit_lang		55		// Zeitdauer fr lang drcken (x 10ms)
#define		DE_Zeit_repeat		10		// Zeitdauer fr Wiederholungen (x 10ms)
volatile uint16_t DE_Status;			// Entprellter und invertierter Status der DE: bit = 1: DE bettigt
#define		Repeat_Maske		(x == 3) || (x == 4) || (x == 5) || (x == 6) || (x == 7)
volatile uint16_t DE_Erkannt;			// Bettigung eines DE erkannt
volatile uint16_t DE_Repeat;			// DE lang Bettigt und Wiederholt
volatile uint16_t DE_Flanke_fall;		// fallende Flanke erkannt
volatile uint16_t DE_Flanke_steig;		// steiende Flanke erkannt
volatile uint16_t DE_Zwei;				// zweifache DE Bettigung erkannt


// Timer
#define Anzahl_Timer	50
volatile uint8_t Timer_Status[Anzahl_Timer];    // 0= Timer aus, 1= Timer luft
volatile uint8_t T_End_Stunde[Anzahl_Timer];    // Timerendwerte
volatile uint8_t T_End_Minute[Anzahl_Timer];
volatile uint8_t T_End_Sekunde[Anzahl_Timer];
volatile uint8_t T_End_10ms[Anzahl_Timer];
volatile uint8_t T_End_Tag[Anzahl_Timer];
volatile uint8_t T_End_Monat[Anzahl_Timer];
volatile uint8_t T_End_Jahr[Anzahl_Timer];


// Dimmer
#define		Timer1_Top	25000			// 100Hz

#define		Dimmer_0		1
#define		Dimmer_0_DA		DA1			// Dimmausgang Dimmer 0
#define		Dimmer_0_Max	100			// Maximalwert
#define		Dimmer_0_Min	50			// Minimalwert

#define		Dimmer_1		0
#define		Dimmer_1_DA					// Dimmausgang Dimmer 1
#define		Dimmer_1_Max	100			// Maximalwert, Dimmwert bei Sonderfall Schlafzimmer
#define		Dimmer_1_Min	50			// Minimalwert

#define		Sonderfall_SZ	0			// Dimmer 1 wird fr beide Nachtkastllampen verwendet, Dimmwert wird fix vergeben
#define		Dimmer_1_DA2				// 2. Dimmausgang fr Nachtkastllampe rechts

volatile uint8_t Dimmer_Soll[3];			// Dimmer Sollwert
volatile uint8_t Dimmer_Ist[3];			// Dimmer Istwert
volatile uint16_t Dimmer_Log[101];		// logarithmische Dimmwerte fr PWM
volatile float Dimmer_Min[3];			// Minimalwerte
volatile float Dimmer_Max[3];			// Maximalwerte
volatile float Dimmer_Faktor[3];		// Faktor zur Min/Max Umrechnung
volatile float Dimmer_Soll_MM_f[3];		// Dimmer Sollwert nach Bercksichtigung der Min/Max Werte
volatile uint8_t Dimmer_Soll_MM[3];
volatile uint8_t Dimmer_Status[3];		// 0=aus, 1=Dimmwert > 0, 10=Fade rauf, 11=Fade runter

#if Sonderfall_SZ
volatile uint8_t Sonderfall_SZ_links;	//  Nachtkastllampe links 0=aus, 1=ein
volatile uint8_t Sonderfall_SZ_rechts;	//	Nachtkastllampe rechts 0=aus, 1=ein
#endif


// Led Modul Ansteuerung
volatile uint8_t Led_Modus[8];		// Betriebsmodus (0=aus, 1=einzelne Ausgnge, 2=Farbe, 3=auto Farbwechsel)
volatile uint8_t Led_Info_1[8];		// Modus 1: DA1 Sollwert, Modus 2: Farbe, Modus 3: Geschwindigkeit
volatile uint8_t Led_Info_2[8];		// Modus 1: DA2 Sollwert, Modus 2&3: Dimmwert
volatile uint8_t Led_Info_3[8];		// Modus 1: DA3 Sollwert
volatile uint8_t Led_Info_4[8];		// Modus 1: DA4 Sollwert


// Allgemeines
volatile uint16_t Helligkeit;		// aktueller Auen-Helligkeitswert	keine Einheit
volatile float Temp_Au;				// aktuelle Auentemperatur			C
volatile uint16_t Luftdruck;		// aktueller Luftdruck				hPa
volatile uint8_t Feuchte_Au;		// aktuelle Auen-Luftfeuchte		%

volatile uint8_t RK_Fenster;		// Reedkontakt Fenster Kche		0=geschlossen, 1=offen


// Rollos
#define		Rollo_0				1		// WZ-MO-Ost
#define		Rollo_0_DA_auf		DA2
#define		Rollo_0_DA_ab		DA3
#define		Rollo_Zeit_0		1900	// in 10ms

#define		Rollo_1				1		// WZ-MO-Sd
#define		Rollo_1_DA_auf		DA4
#define		Rollo_1_DA_ab		DA5
#define		Rollo_Zeit_1		1900	// in 10ms

volatile uint8_t Rollo_Status[2];	// Status vom Rollo: 10=schlieen, 11=schliet, 20=ffnen, 21=ffnet, 0=stopp
volatile uint16_t Rollo_Pos[2];		// 1000=oben/offen, 2900=unten/geschlossen
volatile uint16_t Rollo_Pos_Soll[2];	// Sollposition
volatile uint8_t Rollo_Auto[2];		// Merker fr Automatische Positionsanfahrt


// Prsenzmelder
volatile uint8_t WZ_PM_Bewegung;	// Minuten seit letzter registrierter Bewegung
volatile uint8_t WZ_PM_gesperrt;	// Sekunden; 0=PM wird beachtet; >0 PM wird nicht beachtet
volatile uint8_t WZ_PM_ein;			// PM hat Licht eingeschalten


// Stromstoschalter
#define		USE_SS_DA1		0
#define		USE_SS_DA2		0
#define		USE_SS_DA3		0
#define		USE_SS_DA4		0
#define		USE_SS_DA5		0
#define		USE_SS_DA6		1
#define		USE_SS_DA7		1
#define		USE_SS_DA8		1

#define		SS_DA1_Rueckmeldung		0
#define		SS_DA2_Rueckmeldung		0
#define		SS_DA3_Rueckmeldung		0
#define		SS_DA4_Rueckmeldung		0
#define		SS_DA5_Rueckmeldung		0
#define		SS_DA6_Rueckmeldung		DE9
#define		SS_DA7_Rueckmeldung		DE10
#define		SS_DA8_Rueckmeldung		DE11


// Finster/Hell Umschaltung fr Licht und Rollos
volatile uint8_t Finster;		// 0=hell, 1=Licht an, 2=Rollos runter

#define		Licht_an_Jan_Stu	16
#define		Licht_an_Jan_Min	0
#define		Licht_aus_Jan_Stu	8
#define		Licht_aus_Jan_Min	0
#define		Rollo_zu_Jan_Stu	19
#define		Rollo_zu_Jan_Min	0
#define		Rollo_auf_Jan_Stu	7
#define		Rollo_auf_Jan_Min	0

#define		Licht_an_Feb_Stu	16
#define		Licht_an_Feb_Min	0
#define		Licht_aus_Feb_Stu	8
#define		Licht_aus_Feb_Min	0
#define		Rollo_zu_Feb_Stu	19
#define		Rollo_zu_Feb_Min	0
#define		Rollo_auf_Feb_Stu	7
#define		Rollo_auf_Feb_Min	0

#define		Licht_an_Mar_Stu	17
#define		Licht_an_Mar_Min	0
#define		Licht_aus_Mar_Stu	7
#define		Licht_aus_Mar_Min	0
#define		Rollo_zu_Mar_Stu	19
#define		Rollo_zu_Mar_Min	0
#define		Rollo_auf_Mar_Stu	6
#define		Rollo_auf_Mar_Min	30

#define		Licht_an_Apr_Stu	18
#define		Licht_an_Apr_Min	0
#define		Licht_aus_Apr_Stu	7
#define		Licht_aus_Apr_Min	0
#define		Rollo_zu_Apr_Stu	19
#define		Rollo_zu_Apr_Min	30
#define		Rollo_auf_Apr_Stu	6
#define		Rollo_auf_Apr_Min	30

#define		Licht_an_Mai_Stu	19
#define		Licht_an_Mai_Min	0
#define		Licht_aus_Mai_Stu	7
#define		Licht_aus_Mai_Min	0
#define		Rollo_zu_Mai_Stu	20
#define		Rollo_zu_Mai_Min	0
#define		Rollo_auf_Mai_Stu	6
#define		Rollo_auf_Mai_Min	30

#define		Licht_an_Jun_Stu	20
#define		Licht_an_Jun_Min	0
#define		Licht_aus_Jun_Stu	6
#define		Licht_aus_Jun_Min	0
#define		Rollo_zu_Jun_Stu	20
#define		Rollo_zu_Jun_Min	30
#define		Rollo_auf_Jun_Stu	6
#define		Rollo_auf_Jun_Min	0

#define		Licht_an_Jul_Stu	20
#define		Licht_an_Jul_Min	0
#define		Licht_aus_Jul_Stu	6
#define		Licht_aus_Jul_Min	0
#define		Rollo_zu_Jul_Stu	20
#define		Rollo_zu_Jul_Min	30
#define		Rollo_auf_Jul_Stu	6
#define		Rollo_auf_Jul_Min	0

#define		Licht_an_Aug_Stu	19
#define		Licht_an_Aug_Min	0
#define		Licht_aus_Aug_Stu	6
#define		Licht_aus_Aug_Min	30
#define		Rollo_zu_Aug_Stu	20
#define		Rollo_zu_Aug_Min	0
#define		Rollo_auf_Aug_Stu	6
#define		Rollo_auf_Aug_Min	0

#define		Licht_an_Sep_Stu	18
#define		Licht_an_Sep_Min	0
#define		Licht_aus_Sep_Stu	7
#define		Licht_aus_Sep_Min	0
#define		Rollo_zu_Sep_Stu	20
#define		Rollo_zu_Sep_Min	0
#define		Rollo_auf_Sep_Stu	6
#define		Rollo_auf_Sep_Min	30

#define		Licht_an_Okt_Stu	17
#define		Licht_an_Okt_Min	0
#define		Licht_aus_Okt_Stu	8
#define		Licht_aus_Okt_Min	0
#define		Rollo_zu_Okt_Stu	19
#define		Rollo_zu_Okt_Min	30
#define		Rollo_auf_Okt_Stu	7
#define		Rollo_auf_Okt_Min	0

#define		Licht_an_Nov_Stu	16
#define		Licht_an_Nov_Min	0
#define		Licht_aus_Nov_Stu	8
#define		Licht_aus_Nov_Min	0
#define		Rollo_zu_Nov_Stu	19
#define		Rollo_zu_Nov_Min	0
#define		Rollo_auf_Nov_Stu	7
#define		Rollo_auf_Nov_Min	0

#define		Licht_an_Dez_Stu	16
#define		Licht_an_Dez_Min	0
#define		Licht_aus_Dez_Stu	8
#define		Licht_aus_Dez_Min	0
#define		Rollo_zu_Dez_Stu	19
#define		Rollo_zu_Dez_Min	0
#define		Rollo_auf_Dez_Stu	7
#define		Rollo_auf_Dez_Min	0

#endif // _Config_H