//-------------------------------------------------------------------------------------------------------
//
// Module.c
//
//	- Ansteuerung Stromstoschalter
//	- Timer
//	- Rollosteuerung
//	- Dimmersteuerung
//	- Led-Modul Sollwertvorgabe
//
//-------------------------------------------------------------------------------------------------------

#include "Config.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <math.h>

#include "MCP2515.h"
#include "Error.h"
#include "SPI.h"
#include "EEPROM.h"
#include "DE.h"
#include "Funktionen.h"


//-------------------------------------------------------------------------------------------------------
// Funktionsprototypen

void F_Stromstoss_ein(uint8_t Ausgang);
void F_Stromstoss_aus(uint8_t Ausgang);

void F_Timer_starten(uint8_t T_Nr, uint8_t T_St, uint8_t T_Mi, uint8_t T_Se, uint8_t T_10,
uint8_t T_Ta, uint8_t T_Mo, uint8_t T_Ja);
void F_Timer_stopp(uint8_t T_Nr);
void F_Timer(void);

void F_Rollo_init(void);
void F_Rollo_ab(uint8_t Nr);
void F_Rollo_auf(uint8_t Nr);
void F_Rollo_stopp(uint8_t Nr);
void F_Rollo_Pos(uint8_t Nr, uint8_t Position);
void F_Rollo(void);

void F_Dimmer_init(void);
void F_Dimmer_Soll(uint8_t Dimmer_Nr, uint8_t Sollwert);



/////////////////////////////////////////////////////////////////////////////////////////////////////////



//-------------------------------------------------------------------------------------------------------
// Stromstoschalter einschalten

void F_Stromstoss_ein(uint8_t Ausgang)
{
	switch(Ausgang)
	{
		#if USE_SS_DA1
		case 1:	if(!F_DE_status(SS_DA1_Rueckmeldung))
				{
					SET(DA1);
					F_Timer_starten(0, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA2
		case 2:	if(!F_DE_status(SS_DA2_Rueckmeldung))
				{
					SET(DA2);
					F_Timer_starten(1, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA3
		case 3:	if(!F_DE_status(SS_DA3_Rueckmeldung))
				{
					SET(DA3);
					F_Timer_starten(2, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA4
		case 4:	if(!F_DE_status(SS_DA4_Rueckmeldung))
				{
					SET(DA4);
					F_Timer_starten(3, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA5
		case 5:	if(!F_DE_status(SS_DA5_Rueckmeldung))
				{
					SET(DA5);
					F_Timer_starten(4, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA6
		case 6:	if(!F_DE_status(SS_DA6_Rueckmeldung))
				{
					SET(DA6);
					F_Timer_starten(5, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA7
		case 7:	if(!F_DE_status(SS_DA7_Rueckmeldung))
				{
					SET(DA7);
					F_Timer_starten(6, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA8
		case 8:	if(!F_DE_status(SS_DA8_Rueckmeldung))
				{
					SET(DA8);
					F_Timer_starten(7, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		default: break;
	}
}


//-------------------------------------------------------------------------------------------------------
// Stromstoschalter ausschalten

void F_Stromstoss_aus(uint8_t Ausgang)
{
	switch(Ausgang)
	{
		#if USE_SS_DA1
		case 1:	if(F_DE_status(SS_DA1_Rueckmeldung))
				{
					SET(DA1);
					F_Timer_starten(0, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA2
		case 2:	if(F_DE_status(SS_DA2_Rueckmeldung))
				{
					SET(DA2);
					F_Timer_starten(1, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA3
		case 3:	if(F_DE_status(SS_DA3_Rueckmeldung))
				{
					SET(DA3);
					F_Timer_starten(2, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA4
		case 4:	if(F_DE_status(SS_DA4_Rueckmeldung))
				{
					SET(DA4);
					F_Timer_starten(3, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA5
		case 5:	if(F_DE_status(SS_DA5_Rueckmeldung))
				{
					SET(DA5);
					F_Timer_starten(4, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA6
		case 6:	if(F_DE_status(SS_DA6_Rueckmeldung))
				{
					SET(DA6);
					F_Timer_starten(5, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA7
		case 7:	if(F_DE_status(SS_DA7_Rueckmeldung))
				{
					SET(DA7);
					F_Timer_starten(6, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		#if USE_SS_DA8
		case 8:	if(F_DE_status(SS_DA8_Rueckmeldung))
				{
					SET(DA8);
					F_Timer_starten(7, 0, 0, 0, 10, 0, 0, 0);  // 100ms
				}
				break;
		#endif
		
		default: break;
	}
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////



//-------------------------------------------------------------------------------------------------------
// Timer starten
// F_Timer_starten(1, 0, 0, 10, 0, 0, 0, 0); -> Timer 1: 10 Sekunden

void F_Timer_starten(uint8_t T_Nr, uint8_t T_St, uint8_t T_Mi, uint8_t T_Se, uint8_t T_10,
uint8_t T_Ta, uint8_t T_Mo, uint8_t T_Ja)
{
	// Timer Endwerte speichern

	T_End_Stunde[T_Nr] = Stunde + T_St;
	T_End_Minute[T_Nr] = Minute + T_Mi;
	T_End_Sekunde[T_Nr] = Sekunde + T_Se;
	T_End_10ms[T_Nr] = Ms10 + T_10;
	T_End_Tag[T_Nr] = Tag + T_Ta;
	T_End_Monat[T_Nr] = Monat + T_Mo;
	T_End_Jahr[T_Nr] = Jahr + T_Ja;

	if(T_End_10ms[T_Nr] >= 100)
	{
		T_End_10ms[T_Nr] -= 100;
		T_End_Sekunde[T_Nr]++;
	}

	if(T_End_Sekunde[T_Nr] >= 60)
	{
		T_End_Sekunde[T_Nr] -= 60;
		T_End_Minute[T_Nr]++;
	}

	if(T_End_Minute[T_Nr] >= 60)
	{
		T_End_Minute[T_Nr] -= 60;
		T_End_Stunde[T_Nr]++;
	}

	if(T_End_Stunde[T_Nr] >= 24)
	{
		T_End_Stunde[T_Nr] -= 24;
		T_End_Tag[T_Nr]++;
	}

	if(Monat == 1 || Monat == 3 || Monat == 5 || Monat == 7 || Monat == 8 || Monat == 10 || Monat == 12)
	{
		if(T_End_Tag[T_Nr] > 31)
		{
			T_End_Tag[T_Nr] -= 31;
			T_End_Monat[T_Nr]++;
		}
	}
	else if(Monat == 4 || Monat == 6 || Monat == 9 || Monat == 11)
	{
		if(T_End_Tag[T_Nr] > 30)
		{
			T_End_Tag[T_Nr] -= 30;
			T_End_Monat[T_Nr]++;
		}
	}
	else if(Monat == 2)
	{
		if(Jahr%4 == 0)  // Schaltjahr
		{
			if(T_End_Tag[T_Nr] > 29)
			{
				T_End_Tag[T_Nr] -= 29;
				T_End_Monat[T_Nr]++;
			}
		}
		else
		{
			if(T_End_Tag[T_Nr] > 28)
			{
				T_End_Tag[T_Nr] -= 28;
				T_End_Monat[T_Nr]++;
			}
		}
	}

	if(T_End_Monat[T_Nr] > 12)
	{
		T_End_Monat[T_Nr] -= 12;
		T_End_Jahr[T_Nr]++;
	}
	
	
	Timer_Status[T_Nr] = 1;  // Timer starten
}


//-------------------------------------------------------------------------------------------------------
// laufenden Timer stoppen

void F_Timer_stopp(uint8_t T_Nr)
{
	Timer_Status[T_Nr] = 0;
}



//-------------------------------------------------------------------------------------------------------
// Timer abgelaufen?

void F_Timer(void)
{
	uint8_t i;

	// Timer

	for(i=0; i<Anzahl_Timer; i++)
	{
		if(Timer_Status[i] == 1)
		{
			if((Jahr >= T_End_Jahr[i]) && (Monat >= T_End_Monat[i]) && (Tag >= T_End_Tag[i]) &&
			(Stunde >= T_End_Stunde[i]) && (Minute >= T_End_Minute[i]) &&
			(Sekunde >= T_End_Sekunde[i]) && (Ms10 >= T_End_10ms[i]))
			{
				Timer_Status[i] = 0;  // Timer abgelaufen
				
				F_Timer_abgelaufen(i);
			}
		}
	}
}



/////////////////////////////////////////////////////////////////////////////////////////////////////////



//-------------------------------------------------------------------------------------------------------
// Rollo initialisieren
void F_Rollo_init(void)
{
	#if Rollo_0
		Rollo_Status[0] = 0;
		Rollo_Pos[0] = 1000;								// Annahme offen
	#endif
	
	#if Rollo_1
		Rollo_Status[1] = 0;
		Rollo_Pos[1] = 1000;								// Annahme offen
	#endif	
}


//-------------------------------------------------------------------------------------------------------
// Manuellbetrieb: Rollo schlieen

void F_Rollo_ab(uint8_t Nr)
{
	Rollo_Auto[Nr] = 0;
	Rollo_Status[Nr] = 10;
}


//-------------------------------------------------------------------------------------------------------
// Manuellbetrieb: Rollo ffnen

void F_Rollo_auf(uint8_t Nr)
{
	Rollo_Auto[Nr] = 0;
	Rollo_Status[Nr] = 20;
}


//-------------------------------------------------------------------------------------------------------
// Manuellbetrieb: Rollo stoppen

void F_Rollo_stopp(uint8_t Nr)
{
	Rollo_Auto[Nr] = 0;
	Rollo_Status[Nr] = 0;
}


//-------------------------------------------------------------------------------------------------------
// genaue Position fr Rollo vorgeben
// Position: 0=offen, 100=geschlossen

void F_Rollo_Pos(uint8_t Nr, uint8_t Position)
{	
	Rollo_Pos_Soll[Nr] = (Position * 19) + 1000;
	Rollo_Auto[Nr] = 1;
}


//-------------------------------------------------------------------------------------------------------
// Rollo Ansteuerung und Positionsberwachung

void F_Rollo(void)
{
	static uint8_t Ms10_alt[2];
	uint8_t i;
	
	#if Rollo_0
	switch(Rollo_Status[0])
	{
		case 0:		// Rollo stopp
					RESET(Rollo_0_DA_ab);
					RESET(Rollo_0_DA_auf);
					break;
			
		case 10:	// Rollo schlieen
					RESET(Rollo_0_DA_auf);
					SET(Rollo_0_DA_ab);
					Rollo_Status[0] = 11;	// Rollo schliet
					break;
			
		case 11:	// Rollo schliet
					if(Ms10 != Ms10_alt[0])
					/*{
						Ms10_alt[0] = Ms10;
						Rollo_Pos[0] ++;
					}*/
					break;
			
		case 20:	// Rollo ffnen
					RESET(Rollo_0_DA_ab);
					SET(Rollo_0_DA_auf);
					Rollo_Status[0] = 21;	// Rollo ffnet
					break;
			
		case 21:	// Rollo ffnet
					if(Ms10 != Ms10_alt[0])
					/*{
						Ms10_alt[0] = Ms10;
						Rollo_Pos[0] --;
					}*/
					break;
			
		default: break;
	}
	
	#endif
	
	#if Rollo_1
	switch(Rollo_Status[1])
	{
		case 0:		// Rollo stopp
					RESET(Rollo_1_DA_ab);
					RESET(Rollo_1_DA_auf);
					break;
		
		case 10:	// Rollo schlieen
					RESET(Rollo_1_DA_auf);
					SET(Rollo_1_DA_ab);
					Rollo_Status[1] = 11;	// Rollo schliet
					break;
		
		case 11:	// Rollo schliet
					/*if(Ms10 != Ms10_alt[1])
					{
						Ms10_alt[1] = Ms10;
						Rollo_Pos[1] ++;
					}*/
					break;
		
		case 20:	// Rollo ffnen
					RESET(Rollo_1_DA_ab);
					SET(Rollo_1_DA_auf);
					Rollo_Status[1] = 21;	// Rollo ffnet
					break;
		
		case 21:	// Rollo ffnet
					/*if(Ms10 != Ms10_alt[1])
					{
						Ms10_alt[1] = Ms10;
						Rollo_Pos[1] --;
					}*/
					break;
		
		default: break;
	}
	#endif


	for(i=0; i<2; i++)
	{
		if(Rollo_Auto[i] == 1)  // Automatisch Position anfahren
		{
			if(Rollo_Pos_Soll[i] == 1000)	// ganz ffnen
			{
				F_Rollo_auf(i);
				
				if(i == 0)
					F_Timer_starten(10, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
				
				else if(i == 1)
					F_Timer_starten(9, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
			}
			
			else if(Rollo_Pos_Soll[i] == 2900)	// ganz schlieen
			{
				F_Rollo_ab(i);
				
				if(i == 0)
					F_Timer_starten(10, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
				
				else if(i == 1)
					F_Timer_starten(9, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
			}

			else  // Zwischenwerte anfahren
			{
				if((Rollo_Pos[i] - 10) > Rollo_Pos_Soll[i])
					Rollo_Status[i] = 20;	// ffnen
				
				else if((Rollo_Pos[i] + 10) < Rollo_Pos_Soll[i])
					Rollo_Status[i] = 10;	// schlieen
				
				else
				{
					Rollo_Status[i] = 0;	// stoppen
					Rollo_Auto[i] = 0;
				}
			}
		}

		else  // Manuellbetrieb
		{
			if(Rollo_Pos[i] > 2900)
				Rollo_Pos[i] = 2900;
			
			else if(Rollo_Pos[i] < 1000)
				Rollo_Pos[i] = 1000;
		}
	}	
}




/////////////////////////////////////////////////////////////////////////////////////////////////////////



//-------------------------------------------------------------------------------------------------------
// PWM Dimmer initialisieren
// verwendeter Dimmer: Eltako LUD12-230V
//	- PWM Frequenz = 100Hz
//	- Dimmer aus = 0% PWM
//	- Dimmer 100% ein = 90% PWM

void F_Dimmer_init(void)
{
	uint8_t i=0;


	// Logarithmische Dimmkurve		
	Dimmer_Log[0] = 0;			Dimmer_Log[17] = 69;		Dimmer_Log[34] = 225;
	Dimmer_Log[1] = 23;			Dimmer_Log[18] = 74;		Dimmer_Log[35] = 241;
	Dimmer_Log[2] = 24;			Dimmer_Log[19] = 79;		Dimmer_Log[36] = 259;
	Dimmer_Log[3] = 26;			Dimmer_Log[20] = 85;		Dimmer_Log[37] = 277;
	Dimmer_Log[4] = 28;			Dimmer_Log[21] = 91;		Dimmer_Log[38] = 297;
	Dimmer_Log[5] = 30;			Dimmer_Log[22] = 97;		Dimmer_Log[39] = 319;
	Dimmer_Log[6] = 32;			Dimmer_Log[23] = 104;		Dimmer_Log[40] = 342;
	Dimmer_Log[7] = 34;			Dimmer_Log[24] = 112;		Dimmer_Log[41] = 367;
	Dimmer_Log[8] = 37;			Dimmer_Log[25] = 120;		Dimmer_Log[42] = 393;
	Dimmer_Log[9] = 39;			Dimmer_Log[26] = 129;		Dimmer_Log[43] = 422;
	Dimmer_Log[10] = 42;		Dimmer_Log[27] = 138;		Dimmer_Log[44] = 452;
	Dimmer_Log[11] = 45;		Dimmer_Log[28] = 148;		Dimmer_Log[45] = 485;
	Dimmer_Log[12] = 48;		Dimmer_Log[29] = 159;		Dimmer_Log[46] = 520;
	Dimmer_Log[13] = 52;		Dimmer_Log[30] = 170;		Dimmer_Log[47] = 557;
	Dimmer_Log[14] = 56;		Dimmer_Log[31] = 183;		Dimmer_Log[48] = 598;
	Dimmer_Log[15] = 60;		Dimmer_Log[32] = 196;		Dimmer_Log[49] = 641;
	Dimmer_Log[16] = 64;		Dimmer_Log[33] = 210;		Dimmer_Log[50] = 687;
	
	Dimmer_Log[51] = 737;		Dimmer_Log[68] = 2413;		Dimmer_Log[85] = 7900;
	Dimmer_Log[52] = 790;		Dimmer_Log[69] = 2587;		Dimmer_Log[86] = 8471;
	Dimmer_Log[53] = 847;		Dimmer_Log[70] = 2774;		Dimmer_Log[87] = 9083;
	Dimmer_Log[54] = 908;		Dimmer_Log[71] = 2974;		Dimmer_Log[88] = 9740;
	Dimmer_Log[55] = 974;		Dimmer_Log[72] = 3189;		Dimmer_Log[89] = 10444;
	Dimmer_Log[56] = 1044;		Dimmer_Log[73] = 3420;		Dimmer_Log[90] = 11198;
	Dimmer_Log[57] = 1120;		Dimmer_Log[74] = 3667;		Dimmer_Log[91] = 12008;
	Dimmer_Log[58] = 1201;		Dimmer_Log[75] = 3932;		Dimmer_Log[92] = 12875;
	Dimmer_Log[59] = 1288;		Dimmer_Log[76] = 4216;		Dimmer_Log[93] = 13806;
	Dimmer_Log[60] = 1381;		Dimmer_Log[77] = 4521;		Dimmer_Log[94] = 14804;
	Dimmer_Log[61] = 1480;		Dimmer_Log[78] = 4848;		Dimmer_Log[95] = 15873;
	Dimmer_Log[62] = 1587;		Dimmer_Log[79] = 5198;		Dimmer_Log[96] = 17021;
	Dimmer_Log[63] = 1702;		Dimmer_Log[80] = 5573;		Dimmer_Log[97] = 18251;
	Dimmer_Log[64] = 1825;		Dimmer_Log[81] = 5976;		Dimmer_Log[98] = 19570;
	Dimmer_Log[65] = 1957;		Dimmer_Log[82] = 6408;		Dimmer_Log[99] = 20984;
	Dimmer_Log[66] = 2098;		Dimmer_Log[83] = 6871;		Dimmer_Log[100] = 22500;
	Dimmer_Log[67] = 2250;		Dimmer_Log[84] = 7368;

	// Min/Max Werte definieren
	Dimmer_Max[0] = Dimmer_0_Max;
	Dimmer_Max[1] = Dimmer_1_Max;

	Dimmer_Min[0] = Dimmer_0_Min;
	Dimmer_Min[1] = Dimmer_1_Min;
	
	
	for(i=0; i<2; i++)
	{
		Dimmer_Faktor[i] = (Dimmer_Max[i] - Dimmer_Min[i]) / 100;
	}
	
	
	#if Sonderfall_SZ
	Dimmer_Ist[1] = Dimmer_1_Max;
	OCR1B = Dimmer_Log[Dimmer_Ist[1]];
	#endif

}


//-------------------------------------------------------------------------------------------------------
// Dimmer Sollwert vorgeben
// Dimmer_Nr = 0-1; Sollwert = 0-100
void F_Dimmer_Soll(uint8_t Dimmer_Nr, uint8_t Sollwert)
{
	if(Sollwert == 0)
	{
		Dimmer_Soll[Dimmer_Nr] = 0;
		Dimmer_Soll_MM[Dimmer_Nr] = 0;
	}

	else
	{
		Dimmer_Soll[Dimmer_Nr] = Sollwert;
		Dimmer_Soll_MM_f[Dimmer_Nr] = (float) Dimmer_Soll[Dimmer_Nr] * Dimmer_Faktor[Dimmer_Nr] + Dimmer_Min[Dimmer_Nr] + 0.5;
	
		Dimmer_Soll_MM[Dimmer_Nr] = Dimmer_Soll_MM_f[Dimmer_Nr];
	
		if(Dimmer_Soll_MM[Dimmer_Nr] > Dimmer_Max[Dimmer_Nr])
			Dimmer_Soll_MM[Dimmer_Nr] = Dimmer_Max[Dimmer_Nr];
	
		if(Dimmer_Ist[Dimmer_Nr] < Dimmer_Min[Dimmer_Nr])
			Dimmer_Ist[Dimmer_Nr] = Dimmer_Min[Dimmer_Nr];
	}
}




/////////////////////////////////////////////////////////////////////////////////////////////////////////





//-------------------------------------------------------------------------------------------------------
// Led-Modul Sollwert vorgeben
//
// Modul		LED Modul CAN Adresse
// Modus		Betriebsmodus: 0=aus, 1=einzelne Ausgnge steuern, 2=Farbe, 3=auto Fabwechsel
// Info			Ausgangssollwert, Farbauswahl, Wechselgeschwindigkeit oder Dimmwert

void F_Led_Soll(uint8_t Modul, uint8_t Modus, uint8_t Info_1, uint8_t Info_2, uint8_t Info_3, uint8_t Info_4)
{
	// CAN Nachricht erzeugen
	tCAN message;

	message.id = CAN_Adresse;
	message.header.rtr = 0;
	message.header.length = 7;
	
	
	Led_Modus[Modul-60] = Modus;
	Led_Info_1[Modul-60] = Info_1;
	Led_Info_2[Modul-60] = Info_2;
	Led_Info_3[Modul-60] = Info_3;
	Led_Info_4[Modul-60] = Info_4;


	// CAN Telegramm
	message.data[0] = Modul;
	message.data[1] = 15;		// LED Ansteuern
	message.data[2] = Led_Modus[Modul];
	message.data[3] = Led_Info_1[Modul];
	message.data[4] = Led_Info_2[Modul];
	message.data[5] = Led_Info_3[Modul];
	message.data[6] = Led_Info_4[Modul];

	// CAN Telegramm senden
	while(!F_MCP2515_freier_puffer())
	{
		// auf freien Puffer warten
	}
	F_MCP2515_tel_senden(&message);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////


//-------------------------------------------------------------------------------------------------------
// Finster/Hell Umschaltung

void F_Finster_Hell(void)
{
	switch(Monat)
	{
		case 1:	// Jnner
				if(Stunde >= Licht_an_Jan_Stu && Minute >= Licht_an_Jan_Min && Stunde < Rollo_zu_Jan_Stu && Minute < Rollo_zu_Jan_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Jan_Stu && Minute >= Rollo_zu_Jan_Min) || (Stunde < Rollo_auf_Jan_Stu && Minute < Rollo_auf_Jan_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Jan_Stu && Minute >= Rollo_auf_Jan_Min && Stunde < Licht_aus_Jan_Stu && Minute < Licht_aus_Jan_Min)
					Finster = 1;
				else
					Finster = 0;

		case 2:	// Februar
				if(Stunde >= Licht_an_Feb_Stu && Minute >= Licht_an_Feb_Min && Stunde < Rollo_zu_Feb_Stu && Minute < Rollo_zu_Feb_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Feb_Stu && Minute >= Rollo_zu_Feb_Min) || (Stunde < Rollo_auf_Feb_Stu && Minute < Rollo_auf_Feb_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Feb_Stu && Minute >= Rollo_auf_Feb_Min && Stunde < Licht_aus_Feb_Stu && Minute < Licht_aus_Feb_Min)
					Finster = 1;
				else
					Finster = 0;

		case 3:	// Mrz
				if(Stunde >= Licht_an_Mar_Stu && Minute >= Licht_an_Mar_Min && Stunde < Rollo_zu_Mar_Stu && Minute < Rollo_zu_Mar_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Mar_Stu && Minute >= Rollo_zu_Mar_Min) || (Stunde < Rollo_auf_Mar_Stu && Minute < Rollo_auf_Mar_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Mar_Stu && Minute >= Rollo_auf_Mar_Min && Stunde < Licht_aus_Mar_Stu && Minute < Licht_aus_Mar_Min)
					Finster = 1;
				else
					Finster = 0;

		case 4:	// April
				if(Stunde >= Licht_an_Apr_Stu && Minute >= Licht_an_Apr_Min && Stunde < Rollo_zu_Apr_Stu && Minute < Rollo_zu_Apr_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Apr_Stu && Minute >= Rollo_zu_Apr_Min) || (Stunde < Rollo_auf_Apr_Stu && Minute < Rollo_auf_Apr_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Apr_Stu && Minute >= Rollo_auf_Apr_Min && Stunde < Licht_aus_Apr_Stu && Minute < Licht_aus_Apr_Min)
					Finster = 1;
				else
					Finster = 0;

		case 5:	// Mai
				if(Stunde >= Licht_an_Mai_Stu && Minute >= Licht_an_Mai_Min && Stunde < Rollo_zu_Mai_Stu && Minute < Rollo_zu_Mai_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Mai_Stu && Minute >= Rollo_zu_Mai_Min) || (Stunde < Rollo_auf_Mai_Stu && Minute < Rollo_auf_Mai_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Mai_Stu && Minute >= Rollo_auf_Mai_Min && Stunde < Licht_aus_Mai_Stu && Minute < Licht_aus_Mai_Min)
					Finster = 1;
				else
					Finster = 0;

		case 6:	// Juni
				if(Stunde >= Licht_an_Jun_Stu && Minute >= Licht_an_Jun_Min && Stunde < Rollo_zu_Jun_Stu && Minute < Rollo_zu_Jun_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Jun_Stu && Minute >= Rollo_zu_Jun_Min) || (Stunde < Rollo_auf_Jun_Stu && Minute < Rollo_auf_Jun_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Jun_Stu && Minute >= Rollo_auf_Jun_Min && Stunde < Licht_aus_Jun_Stu && Minute < Licht_aus_Jun_Min)
					Finster = 1;
				else
					Finster = 0;

		case 7:	// Juli
				if(Stunde >= Licht_an_Jul_Stu && Minute >= Licht_an_Jul_Min && Stunde < Rollo_zu_Jul_Stu && Minute < Rollo_zu_Jul_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Jul_Stu && Minute >= Rollo_zu_Jul_Min) || (Stunde < Rollo_auf_Jul_Stu && Minute < Rollo_auf_Jul_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Jul_Stu && Minute >= Rollo_auf_Jul_Min && Stunde < Licht_aus_Jul_Stu && Minute < Licht_aus_Jul_Min)
					Finster = 1;
				else
					Finster = 0;

		case 8:	// August
				if(Stunde >= Licht_an_Aug_Stu && Minute >= Licht_an_Aug_Min && Stunde < Rollo_zu_Aug_Stu && Minute < Rollo_zu_Aug_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Aug_Stu && Minute >= Rollo_zu_Aug_Min) || (Stunde < Rollo_auf_Aug_Stu && Minute < Rollo_auf_Aug_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Aug_Stu && Minute >= Rollo_auf_Aug_Min && Stunde < Licht_aus_Aug_Stu && Minute < Licht_aus_Aug_Min)
					Finster = 1;
				else
					Finster = 0;

		case 9:	// September
				if(Stunde >= Licht_an_Sep_Stu && Minute >= Licht_an_Sep_Min && Stunde < Rollo_zu_Sep_Stu && Minute < Rollo_zu_Sep_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Sep_Stu && Minute >= Rollo_zu_Sep_Min) || (Stunde < Rollo_auf_Sep_Stu && Minute < Rollo_auf_Sep_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Sep_Stu && Minute >= Rollo_auf_Sep_Min && Stunde < Licht_aus_Sep_Stu && Minute < Licht_aus_Sep_Min)
					Finster = 1;
				else
					Finster = 0;

		case 10:	// Oktober
				if(Stunde >= Licht_an_Okt_Stu && Minute >= Licht_an_Okt_Min && Stunde < Rollo_zu_Okt_Stu && Minute < Rollo_zu_Okt_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Okt_Stu && Minute >= Rollo_zu_Okt_Min) || (Stunde < Rollo_auf_Okt_Stu && Minute < Rollo_auf_Okt_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Okt_Stu && Minute >= Rollo_auf_Okt_Min && Stunde < Licht_aus_Okt_Stu && Minute < Licht_aus_Okt_Min)
					Finster = 1;
				else
					Finster = 0;

		case 11:	// November
				if(Stunde >= Licht_an_Nov_Stu && Minute >= Licht_an_Nov_Min && Stunde < Rollo_zu_Nov_Stu && Minute < Rollo_zu_Nov_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Nov_Stu && Minute >= Rollo_zu_Nov_Min) || (Stunde < Rollo_auf_Nov_Stu && Minute < Rollo_auf_Nov_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Nov_Stu && Minute >= Rollo_auf_Nov_Min && Stunde < Licht_aus_Nov_Stu && Minute < Licht_aus_Nov_Min)
					Finster = 1;
				else
					Finster = 0;

		case 12:	// Dezember
				if(Stunde >= Licht_an_Dez_Stu && Minute >= Licht_an_Dez_Min && Stunde < Rollo_zu_Dez_Stu && Minute < Rollo_zu_Dez_Min)
					Finster = 1;
				else if((Stunde >= Rollo_zu_Dez_Stu && Minute >= Rollo_zu_Dez_Min) || (Stunde < Rollo_auf_Dez_Stu && Minute < Rollo_auf_Dez_Min))
					Finster = 2;
				else if(Stunde >= Rollo_auf_Dez_Stu && Minute >= Rollo_auf_Dez_Min && Stunde < Licht_aus_Dez_Stu && Minute < Licht_aus_Dez_Min)
					Finster = 1;
				else
					Finster = 0;

		default: break;
	}
}



