//-------------------------------------------------------------------------------------------------------
//
// Funktionen.c
//
//	- Timer abgelaufen
//	- Ausgnge ber CAN schalten
//	- Initialisierungsfunktionen
//	- Automatikfunktionen
//	- Eingang bettigt
//
//-------------------------------------------------------------------------------------------------------


#include "Config.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "MCP2515.h"
#include "Error.h"
#include "SPI.h"
#include "EEPROM.h"
#include "DE.h"
#include "Module.h"



//-------------------------------------------------------------------------------------------------------
// Timer abgelaufen

void F_Timer_abgelaufen(uint8_t Timer_Nr)
{
	switch(Timer_Nr)
	{	
		// Timer fr Module.c
		
		case 0: // Timer 0: Module.c | F_Stromstoss_ein/aus | DA1
				RESET(DA1);
				break;


		case 1: // Timer 1: Module.c | F_Stromstoss_ein/aus | DA2
				RESET(DA2);
				break;


		case 2: // Timer 2: Module.c | F_Stromstoss_ein/aus | DA3
				RESET(DA3);
				break;


		case 3: // Timer 3: Module.c | F_Stromstoss_ein/aus | DA4
				RESET(DA4);
				break;


		case 4: // Timer 4: Module.c | F_Stromstoss_ein/aus | DA5
				RESET(DA5);
				break;


		case 5: // Timer 5: Module.c | F_Stromstoss_ein/aus | DA6
				RESET(DA6);
				break;


		case 6: // Timer 6: Module.c | F_Stromstoss_ein/aus | DA7
				RESET(DA7);
				break;


		case 7: // Timer 7: Module.c | F_Stromstoss_ein/aus | DA8
				RESET(DA8);
				break;


		case 8: // Timer 8: 
				
				break;
				
				
		case 9: //Timer 9: Funktionen.c | F_DE_betaetigt
				// Rollo 1 stoppen
				RESET(Rollo_1_DA_ab);
				RESET(Rollo_1_DA_auf);
				
				if(Rollo_Status[1] == 11)	// Rollos schlieten
					Rollo_Pos[1] = 2900;
				else if(Rollo_Status[1] == 21)	// Rollos ffneten
					Rollo_Pos[1] = 1000;
				
				Rollo_Status[1] = 0;
				break;


		case 10: //Timer 10: Funktionen.c | F_DE_betaetigt
				// Rollo 0 stoppen
				RESET(Rollo_0_DA_ab);
				RESET(Rollo_0_DA_auf);
				
				if(Rollo_Status[0] == 11)	// Rollos schlieten
					Rollo_Pos[0] = 2900;
				else if(Rollo_Status[0] == 21)	// Rollos ffneten
					Rollo_Pos[0] = 1000;
				
				Rollo_Status[0] = 0;
				break;
		

		// spezifische Timer
		
		case 20:	// Timer 20: Funktionen.c | F_DE_betaetigt | DE6
					F_Stromstoss_aus(5);	// DG-LI ausschalten
					break;
		
		case 21:	// Timer 21: Funktionen.c | F_DE_betaetigt | DE7			
					F_Stromstoss_aus(6);	// SH-LI-EG ausschalten
					break;
		
		case 22:	// Timer 22: Funktionen.c | F_DE_betaetigt | DE8
					F_Stromstoss_aus(7);	// SH-LI-OG ausschalten
					break;

		default: break;
	}
}



/////////////////////////////////////////////////////////////////////////////////////////////////////////




//-------------------------------------------------------------------------------------------------------
// Ausgnge ber CAN ausschalten

void F_Ausgang_ausschalten(uint8_t Ausgang)
{
	switch(Ausgang)
	{
		// Ausgang nicht dimmbar: RESET(DAx);
		// Ausgang mit Dimmer: F_Dimmer_Soll(Dimmer-Nr, 0);
		// Ausgang mit Stromstoschalter: F_Stromstoss_aus(Nr);
		
		case 1: F_Dimmer_Soll(0, 0);
				break;
		case 2: F_Stromstoss_aus(2);
				break;
		case 3: RESET(DA3);
				break;
		case 4: RESET(DA4);
				break;
		case 5: F_Stromstoss_aus(5);
				break;
		case 6: F_Stromstoss_aus(6);
				break;
		case 7: F_Stromstoss_aus(7);
				break;
		case 8: F_Stromstoss_aus(8);
				break;
		default: break;
	}
}



//-------------------------------------------------------------------------------------------------------
// Ausgnge ber CAN einschalten

void F_Ausgang_einschalten(uint8_t Ausgang, uint8_t Dimmwert)
{
	switch(Ausgang)
	{
		// Ausgang nicht dimmbar: SET(DAx);
		// Ausgang mit Dimmer: F_Dimmer_Soll(Dimmer-Nr, Sollwert);
		// Ausgang mit Stromstoschalter: F_Stromstoss_ein(Nr);
		
		case 1:	F_Dimmer_Soll(0, Dimmwert);
				break;
		case 2: F_Stromstoss_ein(2);
				break;
		case 3: SET(DA3);
				break;
		case 4: SET(DA4);
				break;
		case 5: F_Stromstoss_ein(5);
				break;
		case 6: F_Stromstoss_ein(6);
				break;
		case 7: F_Stromstoss_ein(7);
				break;
		case 8: F_Stromstoss_ein(8);
				break;
		default: break;
	}
}



//-------------------------------------------------------------------------------------------------------
// Ausgnge ber CAN toggeln

void F_Ausgang_toggeln(uint8_t Ausgang)
{
	switch(Ausgang)
	{
		case 1:	if(Dimmer_Status[0] == 0 || (Dimmer_Status[0] == 11 && Dimmer_Soll[0] == 0))  // Licht aus
				{
					F_Dimmer_Soll(0, 100);		// K3-LI einschalten
				}
	
				else if(Dimmer_Status[0] > 0)  // Licht an
				{
					F_Dimmer_Soll(0, 0);		// K3-LI ausschalten
				}
				break;

		case 2:	if(F_DE_status(SS_DA2_Rueckmeldung))
				{
					F_Stromstoss_aus(2);		// K3-LI-Bad ausschalten
				}
		
				else
				{
					F_Stromstoss_ein(2);		// K3-LI-Bad einschalten
				}
				break;
		
		case 3: TOGGLE(DA3);
				break;

		case 4: TOGGLE(DA4);
				break;

		case 5: if(F_DE_status(SS_DA5_Rueckmeldung))
				{
					F_Stromstoss_aus(5);		// DG-LI ausschalten
				}
		
				else
				{
					F_Stromstoss_ein(5);		// DG-LI einschalten
				}
				break;

		case 6:	if(F_DE_status(SS_DA6_Rueckmeldung))
				{
					F_Stromstoss_aus(6);		// SH-LI-EG ausschalten
				}
		
				else
				{
					F_Stromstoss_ein(6);		// SH-LI-EG einschalten
				}
				break;

		case 7:	if(F_DE_status(SS_DA7_Rueckmeldung))
				{
					F_Stromstoss_aus(7);		// SH-LI-OG ausschalten
				}
		
				else
				{
					F_Stromstoss_ein(7);		// SH-LI-OG einschalten
				}
				break;

		case 8:	if(F_DE_status(SS_DA8_Rueckmeldung))
				{
					F_Stromstoss_aus(8);		// SH-LI-Keller ausschalten
				}
		
				else
				{
					F_Stromstoss_ein(8);		// SH-LI-Keller einschalten
				}
				break;

		default: break;
	}
}




/////////////////////////////////////////////////////////////////////////////////////////////////////////



//-------------------------------------------------------------------------------------------------------
// Initialisierungsfunktionen

void F_Init(void)
{
	F_Stromstoss_aus(2);	// K3-LI-Bad ausschalten
	F_Stromstoss_aus(5);	// DG-LI ausschalten
	F_Stromstoss_aus(6);	// SH-LI-EG ausschalten
	F_Stromstoss_aus(7);	// SH-LI-OG ausschalten
	F_Stromstoss_aus(8);	// SH-LI-Keller ausschalten

	F_Finster_Hell();
}






/////////////////////////////////////////////////////////////////////////////////////////////////////////


//-------------------------------------------------------------------------------------------------------
// Automatikfunktionen

void F_Auto(void)
{
	static uint8_t Minute_alt;
	static uint8_t Sekunde_alt;
	static uint8_t Finster_alt;
	
	// mintliche Ausfhrung
	if(Minute != Minute_alt  && Sekunde == CAN_Adresse)
	{
		Minute_alt = Minute;
		
		//---------------------------------
		// CON Status ausschicken
		F_CAN_senden(	99, 
				21, 
				(DE_Status & 0x0001) | ((DE_Status & 0x0E00) >> 8) | ((DE_Status & 0x3000) >> 8),
				SH_PM_Bewegung,
				DG_PM_Bewegung,
				Dimmer_Soll[0],
				(Rollo_Pos[0] - 1000) / 19,
				0);
			
		
		//---------------------------------
		// Prsenzmelder: letzte Bewegung
		if(SH_PM_Bewegung <= 254)
			SH_PM_Bewegung++;

		if(DG_PM_Bewegung <= 254)
			DG_PM_Bewegung++;
		
	}
	
	
	
	
	// sekndliche Ausfhrung
	if(Sekunde != Sekunde_alt)
	{
		Sekunde_alt = Sekunde;
		
	}



	// Rollos automatisch ffnen/schlieen
	if(Finster != Finster_alt)
	{
		if(Finster == 1 && Finster_alt == 2)  // Rollos ffnen 
		{
			F_Rollo_auf(0);
			F_Timer_starten(10, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
		}

		else if(Finster == 2)	// Rollos schlieen
		{
			F_Rollo_ab(0);
			F_Timer_starten(10, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
		}

		Finster_alt = Finster;	
	}
}




/////////////////////////////////////////////////////////////////////////////////////////////////////////




//-------------------------------------------------------------------------------------------------------
// Eingang bettigt

void F_DE_betaetigt(void)
{
	static uint8_t M_Dimmer_1, M_Dimmer_2, M_Dimmer_3;
	static uint8_t M_Rollo_3, M_Rollo_4;
				
	// Eingangsauswertung

	// if(F_DE_1kurz(0x0001)) {}
	// if(F_DE_2kurz(0x0001)) {}
	// if(F_DE_lang(0x0001)) {}
	// if(F_DE_fla_fall(0x0001) {}
	// if(F_DE_fla_steig(0x0001) {}
	// if(F_DE_repeat(0x0001) {}
	// bei repeat/lang mssen die Eingnge in der ISR eingetragen werden


	//-------------------------------------------------
	// DE2 - K3-TA Haupt
	if(F_DE_1kurz(DE2))
	{		
		if(Dimmer_Status[0] == 0 || (Dimmer_Status[0] == 11 && Dimmer_Soll[0] == 0))  // Licht aus
		{
			if(M_Dimmer_3 == 0)
				M_Dimmer_3 = 100;
			F_Dimmer_Soll(0, M_Dimmer_3);		// K3-LI einschalten
		}
	
		else if(Dimmer_Status[0] > 0)  // Licht an
		{
			F_Dimmer_Soll(0, 0);		// K3-LI ausschalten
		}
	}
	
	if(F_DE_2kurz(DE2))
	{
		if(Dimmer_Status[0] > 0)	// wenn Licht an, wieder auf 100% dimmen
		{
			M_Dimmer_3 = 100;
			F_Dimmer_Soll(0, M_Dimmer_3);
		}
	}
		
	if(F_DE_repeat(DE2))
	{
		if(M_Dimmer_1 == 0)		// dimmen starten
		{
			M_Dimmer_1 = 1;
			
			if(Dimmer_Ist[0] <= Dimmer_Min[0])
			{
				M_Dimmer_2 = 1;		// rauf dimmen
				Dimmer_Ist[0] = Dimmer_Min[0];
			}
			
			else
				M_Dimmer_2 = 2;		// runter dimmen
			
			M_Dimmer_3 = Dimmer_Ist[0];
		}
		
		if(M_Dimmer_2 == 1)		// rauf dimmen
		{
			M_Dimmer_3++;
			if(M_Dimmer_3 > 100)
			{
				M_Dimmer_3 = 100;
				M_Dimmer_2 = 2;  // runter dimmen
			}
			F_Dimmer_Soll(0, M_Dimmer_3);
		}
		
		else if(M_Dimmer_2 == 2)	// runter dimmen
		{
			if(M_Dimmer_3 >= Dimmer_Min[0] + 5)
				M_Dimmer_3--;
			
			else
			{
				M_Dimmer_3 = Dimmer_Min[0];
				M_Dimmer_2 = 1;	// rauf dimmen
			}
			
			F_Dimmer_Soll(0, M_Dimmer_3);
		}
	}
	
	if(F_DE_fla_fall(DE2))
	{
		if(M_Dimmer_1 == 1)
		{
			M_Dimmer_1 = 0;
		}
	}

	if(F_DE_fla_steig(DE2)) {}


	//-------------------------------------------------
	// DE3 - K3-TA unten
	if(F_DE_1kurz(DE3))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_ab(0);
			F_Timer_starten(10, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
		}
		
		else						// Rollo in Bewegung
		{
			F_Rollo_stopp(0);
			F_Timer_stopp(10);
		}
	}

	if(F_DE_2kurz(DE3)) {}

	if(F_DE_2kurz(DE3)) // halb schlieen
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_ab(0);
			F_Timer_starten(10, 0, 0, 9, 0, 0, 0, 0);	// 9 sek.
		}
		
		else						// Rollo in Bewegung
		{
			F_Rollo_stopp(0);
			F_Timer_stopp(10);
		}
	}
	
	if(F_DE_repeat(DE3))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_ab(0);
			M_Rollo_3 = 1;
		}
	}
	
	if(F_DE_fla_fall(DE3))
	{
		if(M_Rollo_3 == 1)
		{
			M_Rollo_3 = 0;
			F_Rollo_stopp(0);
		}
	}

	if(F_DE_fla_steig(DE3)) {}
	
	
	//-------------------------------------------------
	// DE4 - K3-TA oben
	if(F_DE_1kurz(DE4))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_auf(0);
			F_Timer_starten(10, 0, 0, 20, 0, 0, 0, 0);	// 20 sek.
		}
		
		else						// Rollo in Bewegung
		{
			F_Rollo_stopp(0);
			F_Timer_stopp(10);
		}
	}

	if(F_DE_2kurz(DE4)) {}
	
	if(F_DE_repeat(DE4))
	{
		if(Rollo_Status[0] == 0)	// Rollo steht
		{
			F_Rollo_auf(0);
			M_Rollo_4 = 1;
		}
	}
	
	if(F_DE_fla_fall(DE4))
	{
		if(M_Rollo_4 == 1)
		{
			M_Rollo_4 = 0;
			F_Rollo_stopp(0);
		}
	}

	if(F_DE_fla_steig(DE4)) {}


	//-------------------------------------------------
	// DE5 - K3-TA-Bad
	if(F_DE_1kurz(DE5)) 
	{
		if(F_DE_status(SS_DA2_Rueckmeldung))
		{
			F_Stromstoss_aus(2);		// K3-LI-Bad ausschalten
		}
		
		else
		{
			F_Stromstoss_ein(2);		// K3-LI-Bad einschalten
		}
	}

	if(F_DE_2kurz(DE5)) {}

	if(F_DE_repeat(DE5)) {}

	if(F_DE_fla_fall(DE5)) {}

	if(F_DE_fla_steig(DE5)) {}

	
	//-------------------------------------------------
	// DE6 - DG-PM
	if(F_DE_1kurz(DE6)) {}

	if(F_DE_2kurz(DE6)) {}

	if(F_DE_lang(DE6)) {}
	
	if(F_DE_fla_fall(DE6)) {}

	if(F_DE_fla_steig(DE6))
	{
		F_Stromstoss_ein(5);	// DG-LI einschalten
		F_Timer_starten(20, 0, 0, 10, 0, 0, 0, 0);
		DG_PM_Bewegung = 0;							// Zhler zurck setzen
	}

	
	//-------------------------------------------------
	// DE7 - SH-PM-EG
	if(F_DE_1kurz(DE7)) {}

	if(F_DE_2kurz(DE7)) {}

	if(F_DE_lang(DE7)) {}

	if(F_DE_fla_fall(DE7)) {}

	if(F_DE_fla_steig(DE7))
	{
		if(Finster >= 1)
		{
			F_Stromstoss_ein(6);	// SH-LI-EG einschalten
			F_Timer_starten(21, 0, 0, 30, 0, 0, 0, 0);
			SH_PM_Bewegung = 0;							// Zhler zurck setzen
		}
	}


	//-------------------------------------------------
	// DE8 - SH-PM-OG
	if(F_DE_1kurz(DE8)) {}

	if(F_DE_2kurz(DE8)) {}

	if(F_DE_lang(DE8)) {}

	if(F_DE_fla_fall(DE8)) {}

	if(F_DE_fla_steig(DE8))
	{
		if(Finster >= 1)
		{
			F_Stromstoss_ein(7);	// SH-LI-OG einschalten
			F_Timer_starten(22, 0, 0, 30, 0, 0, 0, 0);
			SH_PM_Bewegung = 0;							// Zhler zurck setzen
		}
	}
	
	
	//-------------------------------------------------
	// DE9 - SH-TA-EG
	if(F_DE_1kurz(DE8)) 
	{
		if(F_DE_status(SS_DA8_Rueckmeldung))
		{
			F_Stromstoss_aus(8);		// SH-LI-Keller ausschalten
		}
		
		else
		{
			F_Stromstoss_ein(8);		// SH-LI-Keller einschalten
		}
	}

	if(F_DE_2kurz(DE8)) {}

	if(F_DE_repeat(DE8)) {}

	if(F_DE_fla_fall(DE8)) {}

	if(F_DE_fla_steig(DE8)) {}
}

