//-------------------------------------------------------------------------------------------------------
//
// Funktion bei Meldung Strom-Aus
//
//-------------------------------------------------------------------------------------------------------


#include "Config.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "MCP2515.h"
#include "Error.h"
#include "SPI.h"
#include "EEPROM.h"
#include "Stromaus.h"
#include "Module.h"


//-------------------------------------------------------------------------------------------------------
// Meldung Stromausfall
void F_Stromaus(void)
{
	uint8_t i=0;
	uint8_t j=0;
	
	// CAN Nachricht erzeugen
	tCAN message_rec;			// zum Empfangen
	
	TIMSK0 &= ~(1<<TOIE0);	// Timer 0 stoppen
	TIMSK1 &= ~(1<<TOIE1);	// Timer 1 stoppen
	RESET(LED_G);
	SET(LED_R);
	
	F_EEPROM_schreiben(0, 0, 1);  // Strom Aus Merker setzen
	
	F_EEPROM_schreiben(0, 10, Minute);
	F_EEPROM_schreiben(0, 11, Stunde);
	F_EEPROM_schreiben(0, 12, Tag);
	F_EEPROM_schreiben(0, 13, Monat);
	F_EEPROM_schreiben(0, 14, Jahr);
	
	F_EEPROM_schreiben(0, 20, PIND);			// Status Ausgnge
	F_EEPROM_schreiben(0, 21, Dimmer_Soll[0]);	// Dimmer 1 Sollwert
	F_EEPROM_schreiben(0, 22, Dimmer_Soll[1]);	// Dimmer 2 Sollwert
	F_EEPROM_schreiben(0, 23, Dimmer_Soll[2]);	// Dimmer 3 Sollwert
	F_EEPROM_schreiben(0, 24, Dimmer_Soll[3]);	// Dimmer 4 Sollwert
	
	// Ausgnge rcksetzen
	RESET(DA1);
	RESET(DA2);
	RESET(DA3);
	RESET(DA4);
	RESET(DA5);
	RESET(DA6);
	RESET(DA7);
	RESET(DA8);
	
	// aktive Timer speichern
	for(i=0; i<Anzahl_Timer; i++)
	{
		if(Timer_Status[i] == 1)  // Timer (i+1) ist aktiv
		{
			F_EEPROM_schreiben(0, 30 + j, i + 1);
			F_EEPROM_schreiben(0, 31 + j, *T_End_Jahr);
			F_EEPROM_schreiben(0, 32 + j, *T_End_Monat);
			F_EEPROM_schreiben(0, 33 + j, *T_End_Tag);
			F_EEPROM_schreiben(0, 34 + j, *T_End_Stunde);
			F_EEPROM_schreiben(0, 35 + j, *T_End_Minute);
			F_EEPROM_schreiben(0, 36 + j, *T_End_Sekunde);
			F_EEPROM_schreiben(0, 37 + j, *T_End_10ms);
			
			j += 8;
		}
	}
	
	j = j / 8;  // Anzahl der gespeicherten Timer
		F_EEPROM_schreiben(0, 29, j);
	
	
	while(1)  // auf Stromaus oder Softwarereset warten
	{
		// CAN Telegramm empfangen
		if(MCP2515_tel_empfangen > 0)  // CAN Telegramm empfangen
		{
			F_MCP2515_tel_empfangen(&message_rec);
			MCP2515_tel_empfangen--;

			if(message_rec.data[0] == CAN_Adresse || message_rec.data[0] == 99)  // Prfen, ob die Adresse passt
			{
				if(message_rec.data[1] == 70)		// Steuerbefehle
				{
					if(message_rec.data[2] == 1)	// Softwarereset
					{
						cli();
						wdt_enable(WDTO_15MS);
						while(1);
					}
				}
			}
		}
	}
}

